#!/usr/bin/env python

import os
from cStringIO import StringIO
from unittest import TestCase

# XXX
# Iterators.
# Implicit end of various elements.
# COL and COLGROUP nr columns calculation.
# COL and COLGROUP span attrs.
# is_header for TableColumns returned from HTMLTable.
# Multiple tables, including restricting parsing to first N tables
#  (nr_to_parse, nr_toplevel_to_parse args to HTMLTable).
# recode_entities, single_span and collapse_whitespace args to HTMLTable.
# HTMLTable.get_row_by_name, .get_row_by_nr, .get_col_by_name,
#  .get_col_by_nr
# TableRow.get_cell_by_name, .get_cell_by_nr
# TableColumn.get_cell_by_name, .get_cell_by_nr
# Illegally overlapping spans (rows and rows, cols and cols, rows and
#  cols).

import sys
if sys.version[:5] == "1.5.2":
    PYTHON_VERSION = 152
else:
    PYTHON_VERSION = reduce(lambda x, y: 10*x+y, sys.version_info[:3])


class ModuleTests(TestCase):
    def testSimple(self):
        from ClientTable import ParseFile
        fh = open(os.path.join("testdata", "simple.html"))
        tables = ParseFile(fh, collapse_whitespace=1, strip_tags=1)
        assert len(tables) == 1
        table = tables[0]
        headers = ["Name", "Cups", "Type of Coffee", "Sugar?", "None"]
        for i in range(len(table.headers_row)):
            assert str(table.headers_row[i]) == headers[i]
        assert table.headers_row[-1] is None
        assert table[0].is_header
        row = table[1]
        col = table["Name"]
        cell = col[1]
        cell2 = row["Name"]
        cell3 = row.get_cell_by_nr(0)
        assert str(cell) == str(cell2) == str(cell3) == "T. Sexton"
        assert str(table["Name"][1]) == str(table[1]["Name"]) == "T. Sexton"

    def testStripTags(self):
        from ClientTable import ParseFile
        fh = open(os.path.join("testdata", "simple.html"))
        tables = ParseFile(fh, collapse_whitespace=1, strip_tags=0)
        assert str(tables[0][2][0]) == "<b>J. Dinnen</b>"

        fh = open(os.path.join("testdata", "merged.html"))
        tables = ParseFile(fh, collapse_whitespace=1)
        assert len(tables) == 1
        table = tables[0]
        assert str(table[0][0]) == ""
        assert str(table[0][1]) == "Average"
        assert str(table[0][2]) == "Average"
        assert str(table[0][3]) == "Red<BR>eyes"
        assert str(table[1][0]) == ""
        assert str(table[1][1]) == "height"
        assert str(table[1][2]) == "weight"
        assert str(table[1][3]) == "Red<BR>eyes"
        assert str(table[2][0]) == "Males"
        assert str(table[2][1]) == "1.9"
        assert str(table[2][2]) == "0.003"
        assert str(table[2][3]) == "40%"
        assert str(table[3][0]) == "Females"
        assert str(table[3][1]) == "1.7"
        assert str(table[3][2]) == "0.002"
        assert str(table[3][3]) == "43%"
        assert table[0][1] == table[0][2]
        assert table[0][0] == table[1][0]
        assert table[0][3] == table[1][3]

    def testColSpan(self):
        from ClientTable import ParseFile
        fh = open(os.path.join("testdata", "colspan.html"))
        tables = ParseFile(fh, collapse_whitespace=1)
        assert len(tables) == 1
        table = tables[0]
        assert table.headers_row is None
        assert not table[0].is_header
        assert str(table[0][0]) == "1"
        assert str(table[0][1]) == "2"
        assert str(table[0][2]) == "3"
        assert str(table[1][0]) == "4"
        assert str(table[1][1]) == "4"
        assert str(table[1][2]) == "6"
        assert str(table[2][0]) == "7"
        assert str(table[2][1]) == "8"
        assert str(table[2][2]) == "9"
        assert table[1][0] is table[1][1]

    def testColSpan2(self):
        from ClientTable import ParseFile
        fh = open(os.path.join("testdata", "colspan2.html"))
        tables = ParseFile(fh, collapse_whitespace=1, strip_tags=1)
        assert len(tables) == 1
        table = tables[0]
        headers = ["Name", "Cups", "Type of Coffee", "Sugar?", "None"]
        for i in range(len(table.headers_row)):
            assert str(table.headers_row[i]) == headers[i]
        assert table.headers_row[-1] is None
        assert table[0].is_header
        assert str(table[0][0]) == "Name"
        assert str(table[0][1]) == "Cups"
        assert str(table[0][2]) == "Type of Coffee"
        assert str(table[0][3]) == "Sugar?"
        assert table[0][4] is None
        assert str(table[1][0]) == "T. Sexton"
        assert str(table[1][1]) == "10"
        assert str(table[1][2]) == "Espresso"
        assert str(table[1][3]) == "No"
        assert str(table[1][4]) == "Extra column"
        assert str(table[2][0]) == "J. Dinnen"
        assert str(table[2][1]) == "5"
        assert str(table[2][2]) == "Decaf"
        assert str(table[2][3]) == "Yes"
        assert table[2][4] is None
        assert str(table[3][0]) == "A. Soria"
        assert str(table[3][1]) == "Not available"
        assert str(table[3][2]) == "Not available"
        assert str(table[3][3]) == "Not available"
        assert table[3][4] is None
        assert table[3][1] is table[3][2] is table[3][3]

    def testRowSpan(self):
        from ClientTable import ParseFile
        fh = open(os.path.join("testdata", "rowspan.html"))
        tables = ParseFile(fh, collapse_whitespace=1)
        assert len(tables) == 1
        table = tables[0]
        assert table.headers_row is None
        assert str(table[0][0]) == "1"
        assert str(table[0][1]) == "2"
        assert str(table[0][2]) == "3"
        assert str(table[1][0]) == "4"
        assert str(table[1][1]) == "2"
        assert str(table[1][2]) == "6"
        assert str(table[2][0]) == "7"
        assert str(table[2][1]) == "8"
        assert str(table[2][2]) == "9"
        assert table[0][1] is table[1][1]

    def testColAndRowSpan(self):
        from ClientTable import ParseFile
        fh = open(os.path.join("testdata", "merged.html"))
        tables = ParseFile(fh, collapse_whitespace=1, strip_tags=1)
        assert len(tables) == 1
        table = tables[0]
        assert str(table[0][0]) == ""
        assert str(table[0][1]) == "Average"
        assert str(table[0][2]) == "Average"
        assert str(table[0][3]) == "Red eyes"
        assert str(table[1][0]) == ""
        assert str(table[1][1]) == "height"
        assert str(table[1][2]) == "weight"
        assert str(table[1][3]) == "Red eyes"
        assert str(table[2][0]) == "Males"
        assert str(table[2][1]) == "1.9"
        assert str(table[2][2]) == "0.003"
        assert str(table[2][3]) == "40%"
        assert str(table[3][0]) == "Females"
        assert str(table[3][1]) == "1.7"
        assert str(table[3][2]) == "0.002"
        assert str(table[3][3]) == "43%"
        assert table[0][1] == table[0][2]
        assert table[0][0] == table[1][0]
        assert table[0][3] == table[1][3]

    def testTHTDRow(self):
        from ClientTable import ParseFile
        fh = open(os.path.join("testdata", "th_td_row.html"))
        tables = ParseFile(fh, collapse_whitespace=1)
        assert len(tables) == 1
        table = tables[0]

        col = table.get_col_by_name("Name")
        row = table.get_row_by_name("Alice")
        row2 = table.get_row_by_name("Name")
        assert str(col[0]) == "Name"
        assert str(col[1]) == "Alice"
        assert str(col[2]) == "Bob"
        assert str(row[0]) == "Alice"
        assert str(row[1]) == "20"
        assert str(row2[0]) == "Name"
        assert str(row2[1]) == "Widgets"

        assert str(table[0][0]) == "Name"
        assert str(table[0][1]) == "Widgets"
        assert str(table[1][0]) == "Alice"
        assert str(table[1][1]) == "20"
        assert str(table[2][0]) == "Bob"
        assert str(table[2][1]) == "15"

    def testNested(self):
        from ClientTable import ParseFile, HTMLTable
        fh = open(os.path.join("testdata", "nested.html"))
        tables = ParseFile(fh, collapse_whitespace=1)
        assert len(tables) == 1

        table = tables[0]
        assert len(table) == 2
        assert len(table[0]) == 6
        assert str(table[0][0]) == "a"
        assert str(table[0][1]) == "b"
        assert isinstance(table[0][2], HTMLTable)
        assert isinstance(table[0][3], HTMLTable)
        assert str(table[0][4]) == "c"
        assert str(table[0][5]) == "d"

        assert len(table[1]) == 6
        assert table[1][1] is None
        table1 = table[1][0]
        assert isinstance(table1, HTMLTable)
        assert len(table1) == 1
        assert len(table1[0]) == 1
        assert str(table1[0][0]) == "7"

        table2 = table[0][2]  # 1st row, 3rd object
        assert len(table2) == 1
        assert len(table2[0]) == 3
        assert str(table2[0][0]) == "1"
        assert str(table2[0][1]) == "2"
        assert str(table2[0][2]) == "3"

        table3 = table[0][3]
        assert len(table3) == 1
        assert len(table3[0]) == 4
        assert str(table3[0][0]) == "4"
        assert isinstance(table3[0][1], HTMLTable)
        assert str(table3[0][2]) == "5"
        assert str(table3[0][3]) == "6"

        table4 = table3[0][1]
        assert len(table4) == 1
        assert len(table4[0]) == 2
        assert str(table4[0][0]) == "i"
        assert str(table4[0][1]) == "ii"

    def testIterators(self):
        try:
            StopIteration
        except NameError:
            return

        from ClientTable import ParseFile
        fh = open(os.path.join("testdata", "colspan.html"))
        table = ParseFile(fh, collapse_whitespace=1)[0]

        rows = [("1", "2", "3"),
                ("4", "4", "6"),
                ("7", "8", "9")]

        i = 0
        for row in table:
            for j in range(3):
                assert str(row[j]) == rows[i][j]
            i += 1

        i = j = 0
        for row in table:
            for cell in row:  # sequence rather than iterator
                assert str(cell) == rows[i][j]
                j += 1
            i += 1
            j = 0

        j = i = 0
        for col in table.col_iter():
            for cell in col:  # sequence rather than iterator
                assert str(cell) == rows[i][j]
                i += 1
            j += 1
            i = 0


if __name__ == "__main__":
    import unittest
    unittest.main()
